import numpy as np
import PIL.Image


def rgb2gray(rgb):
    assert rgb.ndim == 3
    gray = PIL.Image.fromarray(rgb)
    gray = gray.convert('L')
    gray = np.asarray(gray)
    return gray


def gray2rgb(gray):
    assert gray.ndim == 2
    rgb = gray[:, :, None].repeat(3, axis=2)
    return rgb


def rgb2rgba(rgb):
    assert rgb.ndim == 3
    a = np.full(rgb.shape[:2], 255, dtype=np.uint8)
    rgba = np.dstack((rgb, a))
    return rgba



# flake8: noqa

import os.path as osp

import numpy as np


here = osp.dirname(osp.abspath(__file__))


def arc2017():
    data_file = osp.join(here, '1532900700692405455.npz')
    data = np.load(data_file)
    return dict(data)



import warnings

import matplotlib.cm
import numpy as np


def depth2rgb(
    depth,
    min_value=None,
    max_value=None,
    colormap='jet',
):
    if min_value is None:
        min_value = np.nanmin(depth)
    if max_value is None:
        max_value = np.nanmax(depth)

    if np.isinf(min_value) or np.isinf(max_value):
        warnings.warn('Min or max value for depth colorization is inf.')
    if max_value == min_value:
        eps = np.finfo(depth.dtype).eps
        max_value += eps
        min_value -= eps

    rgb = np.zeros(depth.shape, dtype=float)

    isnan = np.isnan(depth)
    rgb[~isnan] = 1. * (depth[~isnan] - min_value) / (max_value - min_value)

    colormap_func = getattr(matplotlib.cm, colormap)
    rgb = colormap_func(rgb)[:, :, :3]
    rgb = (rgb * 255).round().astype(np.uint8)
    rgb[isnan] = (0, 0, 0)

    return rgb



try:
    import cv2
except ImportError:
    cv2 = None

import numpy as np
import PIL.Image


def resize(
    src,
    height,
    width,
    interpolation='linear',
    backend='auto',
):
    if not isinstance(src, np.ndarray):
        raise TypeError('src type must be numpy.ndarray')

    if backend == 'auto':
        backend = 'pillow' if cv2 is None else 'opencv'

    if backend == 'pillow':
        if interpolation == 'linear':
            interpolation = PIL.Image.LINEAR
        elif interpolation == 'nearest':
            interpolation = PIL.Image.NEAREST
        else:
            raise ValueError(
                'unsupported interpolation: {}'.format(interpolation)
            )

        src = PIL.Image.fromarray(src)
        dst = src.resize((width, height), interpolation)
        dst = np.asarray(dst)
    elif backend == 'opencv':
        if interpolation == 'linear':
            interpolation = cv2.INTER_LINEAR
        elif interpolation == 'nearest':
            interpolation = cv2.INTER_NEAREST
        else:
            raise ValueError(
                'unsupported interpolation: {}'.format(interpolation)
            )

        dst = cv2.resize(src, (width, height), interpolation=interpolation)
    else:
        raise ValueError('unsupported backend: {}'.format(backend))

    return dst



#!/usr/bin/env python

from __future__ import print_function

import distutils.spawn
from setuptools import find_packages
from setuptools import setup
import shlex
import subprocess
import sys


version = '0.0.3'


if sys.argv[1] == 'release':
    if not distutils.spawn.find_executable('twine'):
        print(
            'Please install twine:\n\n\tpip install twine\n',
            file=sys.stderr,
        )
        sys.exit(1)

    commands = [
        'git tag v{:s}'.format(version),
        'git push origin master --tag',
        'python setup.py sdist',
        'twine upload dist/imgviz-{:s}.tar.gz'.format(version),
    ]
    for cmd in commands:
        subprocess.check_call(shlex.split(cmd))
    sys.exit(0)


install_requires = []
with open('requirements.txt') as f:
    for req in f:
        install_requires.append(req.strip())


setup(
    name='imgviz',
    version=version,
    packages=find_packages(),
    install_requires=install_requires,
    description='Image Visualization Tools',
    author='Kentaro Wada',
    author_email='www.kentaro.wada@gmail.com',
    url='http://github.com/wkentaro/imgviz',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
)



import numpy as np
import PIL.Image
import PIL.ImageDraw


def rectangle(src, aabb1, aabb2, color, fill=None, width=0):
    color = tuple(color)

    src_pil = PIL.Image.fromarray(src)
    draw = PIL.ImageDraw.ImageDraw(src_pil)

    y1, x1 = aabb1
    y2, x2 = aabb2
    draw.rectangle(xy=(x1, y1, x2, y2), fill=fill, outline=color, width=width)

    dst = np.asarray(src_pil)
    return dst



import math

import numpy as np

from .centerize import centerize
from .color import gray2rgb
from .color import rgb2rgba
from .draw import rectangle


def _tile(imgs, shape, dst):
    """Tile images which have same size.

    Parameters
    ----------
    imgs: numpy.ndarray
        Image list which should be tiled.
    shape: tuple of int
        Tile shape.
    dst:
        Image to put the tile on.
    """
    y_num, x_num = shape
    tile_w = imgs[0].shape[1]
    tile_h = imgs[0].shape[0]
    for y in range(y_num):
        for x in range(x_num):
            i = x + y * x_num
            if i < len(imgs):
                y1 = y * tile_h
                y2 = (y + 1) * tile_h
                x1 = x * tile_w
                x2 = (x + 1) * tile_w
                dst[y1:y2, x1:x2] = imgs[i]
    return dst


def _get_tile_shape(num):
    x_num = int(math.sqrt(num))  # floor
    y_num = 0
    while x_num * y_num < num:
        y_num += 1
    return x_num, y_num


def tile(
    imgs,
    shape=None,
    cval=None,
    border=None,
):
    assert isinstance(imgs, (list, tuple))
    imgs = imgs[:]

    if shape is None:
        shape = _get_tile_shape(len(imgs))

    if border:
        border = np.asarray(border, dtype=np.uint8)
        if border.ndim == 1:
            border = border[None].repeat(len(imgs), axis=0)
        assert border.ndim == 2
    else:
        border = (border,) * len(imgs)

    # get max tile size to which each image should be resized
    max_h, max_w = np.inf, np.inf
    for img in imgs:
        max_h = min(max_h, img.shape[0])
        max_w = min(max_w, img.shape[1])

    ndim = max(img.ndim for img in imgs)
    assert ndim in [2, 3]
    channel = max(img.shape[2] for img in imgs if img.ndim == 3)

    # tile images
    for i, img in enumerate(imgs):
        assert img.dtype == np.uint8

        if ndim == 3 and img.ndim == 2:
            img = gray2rgb(img)
        if channel == 4:
            img = rgb2rgba(img)

        img = centerize(src=img, shape=(max_h, max_w, channel), cval=cval)
        if border[i] is not None:
            img = rectangle(
                src=img,
                aabb1=(3, 3),
                aabb2=(img.shape[0] - 3, img.shape[1] - 3),
                color=border[i],
                width=6,
            )
        imgs[i] = img

    height = max_h * shape[0]
    width = max_w * shape[1]
    dst = np.zeros((height, width, channel), dtype=np.uint8)
    if cval:
        dst[:, :] = cval

    return _tile(imgs=imgs, shape=shape, dst=dst)



# flake8: noqa

from . import data

# -----------------------------------------------------------------------------

from .centerize import centerize

from .color import rgb2gray
from .color import gray2rgb

from .colorize import depth2rgb

from .draw import rectangle

from .resize import resize

from .tile import tile
